#!/etc/opt/openit/perl
#
# Patch to add on new data types at end of acc_types
#
# Run in patch.cfg with
#
#   run:update_acc_types.pl:License.logfile.events:files/acc_types.add
#
# (data type is matched by regexp so the . will match space/^ in acc_types
#

use strict;

my $cfg = "/var/opt/openit/etc/acc_types";
my $old = "$cfg.old";
my $new = "$cfg.new";

# Only root can patch
if ( $< ne 0 ) {
    die "Error: Patching must be done as 'root'!\n";
}

open(IN, "$cfg") || die "Error: Unable to read old cfg '$cfg'!\n";
my @lines = <IN>;
close(IN);

my $ind = 0;

#update License logfile events v2
$ind++ while ( $lines[$ind] !~ /License.logfile.events.v2/);
$ind++ while ( $lines[$ind] !~ /Feature.version/);
$ind++;

if ( $lines[$ind] !~ /Used.licenses/) {
	splice ( @lines, $ind , 0 , "   Used^licenses,\n" );
}
$ind+=5;
$lines[$ind] = "   Details)),\n";

#   (Individual^license^use^licenseevents,
$ind = 0;
$ind++ while ( $lines[$ind] !~ /Individual.license.use.licenseevents/ );
$ind++ while ( $lines[$ind] !~ /Prime.time/ );
$ind++;
$lines[$ind] = "   Group^type,\n";
$ind++;
$lines[$ind] = "   Group,\n";
$ind++;
$lines[$ind] = "   User^name)\n";
$ind++;
$lines[$ind] = "  ),\n";
$ind ++ while ( $lines[$ind] !~ /Usergroup.license.use.licenseevents/ );
$ind++ while ( $lines[$ind] !~ /Prime.time/ );
$ind++;
$lines[$ind] = "   Group^type,\n";
$ind++;
$lines[$ind] = "   Group)\n";
$ind++;
$lines[$ind] = " ), \n";


#update Denied.licensevents.logfile
$ind = 0;
if ( ! grep {/Denied.licenseevents.logfile/} @lines ) {
	DEBUG( "Denied.licenseevents.logfile not found. No need to fix datatype name." );
}
else
{
	$ind ++ while ( $lines[$ind] !~ /Denied.licenseevents.logfile/ );
	$lines[$ind] = "  (Denied^license^use^licenseevents,";
}

$ind = 0;
if ( grep {/Denied.license.use.licenseevents/} @lines ) {
	DEBUG ( "Denied.license.use.licenseevents already present." );
}
else
{
	$ind = 0;
	$ind++ while ( $lines[$ind] !~ /Petrel/ );
	$ind++ while ( $lines[$ind] !~ /Petrel/ );
	
	$ind++ while ( $lines[$ind] !~ /Used.licenses/ ); 
		
	$lines[$ind] = "   Used^licenses)\n";
	$ind++;
	$lines[$ind] = "  ),\n\n";
	$ind++;
	my @insert = (  
" (Denied^license^use^licenseevents,\n",
"  (\n",
"   (Count,Record^Count,x,SUM)\n",
"  ),\n",
"  (Product,\n",
"   Feature,\n",
"   Feature^version,\n",
"   User^name,\n",
"   Host^name,\n",
"   Status,\n",
"   Prime^time))\n",
")") ;
	splice ( @lines, $ind , 0 , @insert );
	
}
#
# insert Host.license.use.licenseevents 
if ( ! grep {/Host.license.use.licenseevents/} @lines )
{
	
	$ind = 0;
	$ind++ while ( $lines[$ind] !~ /Denied.license.use.licenseevents/);
	$ind++ while ( $lines[$ind] !~ /Prime.time/);
	$lines[$ind] = "   Prime^time)\n";
	$ind++;
	$lines[$ind] = "  ),\n";
	$ind++;
	
	my @insert = (
"\n (Host^license^use^licenseevents,\n",
"  (\n",
"   (Count,Record^Count,x,SUM),\n",
"   (Elapsed^time,Accumulated^time^for^host^in^hours,Hour,SUM),\n",
"   (Max^used,Maximum^concurrent^licenses^used^by^host,x,MAX),\n",
"   (Distinct,Always^one^for^each^extracted^unique^classification,x,IDENT)\n",
"  ),\n ",
"  (Product,\n",
"   Feature,\n",
"   Feature^version,\n",
"   Prime^time,\n",
"   Group^type,\n",
"   Group,\n",
"   Host^name)\n",
"  ),\n",
"\n",
" (Hostgroup^license^use^licenseevents,\n",
"  (\n",
"   (Count,Record^Count,x,SUM),\n",
"   (Elapsed^time,Accumulated^time^for^hostgroup^in^hours,Hour,SUM),\n",
"   (Avg^used,Average^concurrent^licenses^used^by^hostgroup,x,ABSAVG),\n",
"   (Max^used,Maximum^concurrent^licenses^used^by^hostgroup,x,MAX),\n",
"   (Min^used,Minimum^concurrent^licenses^used^by^hostgroup,x,MIN)\n",
"  ),\n",
"  (Product,\n",
"   Feature,\n",
"   Feature^version,\n",
"   Prime^time,\n",
"   Group^type,\n",
"   Group)\n",
"  )\n" );
	splice ( @lines, $ind , 0 , @insert );
}
else
{
	VERBOSE ( "Host.license.use.licenseevents is already present" );
}


$ind = 0;
while($lines[$ind] !~ /License.logfile.events/ ) { 
    $ind++;
}
while($lines[$ind] !~ /Details/ ) {
    $ind++;
}
$ind++;
#$lines[$ind] =~ s/\)/,/;  
$lines[$ind] = "   Forward^product)\n";
#splice(@lines, $ind, 0, "   Forward^product)\n");




# Write new acc_types 
open(OUT, ">$new") || die "Error: Can't write new cfg '$new'!\n";
print OUT @lines;
close(OUT);

if( ! rename($cfg, $old)) {
    die "Error: Unable to previous config to '$old'!\n";
}

if( ! rename($new, $cfg)) {
    die "Error: Unable to rename new config to '$cfg'!\n";
}

exit 0;


sub VERBOSE
{
	my ( $message ) = @_;
	print " -- " . $message . "\n";
}
sub DEBUG
{
	my ( $message ) = @_;
	print " -- " . $message . "\n";
}
