#!/etc/opt/openit/perl
#
#
#
#
#
#
use strict;
use Getopt::Long;
my $filtdir = _get_config_value( "CONFIG_DIR" ) . "/filtrator/";

my $opt_verbose;
my $opt_debug; 
my $opt_dryrun;
my $opt_help;

unless ( GetOptions("verbose"		=> \ $opt_verbose,
					"debug"		  => \ $opt_debug,
					"dryrun"		=> \ $opt_dryrun,
					"help"		   => \ $opt_help)) {
	&print_usage;
}

my $filtrator_dir = _get_config_value("CONFIG_DIR");
$filtrator_dir = "$filtrator_dir/filtrator";


my @insert_duplicate_license_usage = (
"		Duplicate:license-usage()=\"\"\n",
"		{\n",
"			filter(String)=\"Duplicate\"{}\n",
"			global()=\"\"\n",
"			{\n",
"				set-values(String)=\"feature_version='all'\"{}\n",
"			}\n",
"		}\n"
);

my @insert_duplicate_usergroups = (
"		Duplicate:usergroups()=\"\"\n",
"		{\n",
"			filter(String)=\"Duplicate\"{}\n",
"			limited()=\"\"\n",
"			{\n",
"				map-file-v2(FileName)=\"\${OpeniT.directories.conf}user-to-groups.map\"{}\n",
"				source(String)=\"user\"{}\n",
"				target(String)=\"var9,var8\"{}\n",
"			}\n",
"		}\n"
);

my @insert_threshold_zero_duration = (
"		Threshold:zero-duration()=\"\"\n",
"		{\n",
"			filter(String)=\"Threshold\"{}\n",
"			index(String)=\"duration\"{}\n",
"			value(Real)=\"0\"{}\n",
"			method(String)=\"gt\"{}\n",
"			set-value(String)=\"#1:duration=duration/1000\"{}\n",
"		}\n",
);

&_update_licenseevents_total_usage();
&_update_licenseevents_hostgroups();
&_update_licenseevents_host_usage();
&_update_licenseevents_individual_usage();
&_update_licenseevents_usergroups();
&_update_licenseevents_log_v2();

sub _update_licenseevents_log_v2
{
	my $cfg = $filtrator_dir . "/licenseevents-log-v2.conf"  ;
	my @lines = _read_file( $cfg );
	
	my @insert_def = (
	"		Threshold:empty-event()=\"\"\n",
	"		{\n",
	"			filter(String)=\"Threshold\"{}\n",
	"			limiting(String)=\"\$event_type == ' '\"{}\n",
	"			index(String)=\"event_type\"{}\n",
	"			value(String)=\"\"{}\n",
	"			method(String)=\"seq\"{}\n",
	"		}\n",	
	);
	
	&_add_filter_process( \@lines, \@insert_def , "" , "Threshold:empty-event" , 1 );
	
	@insert_def = ();
	@insert_def = (
	"		Mapping:status()=\"\"\n",
	"		{\n",
	"			filter(String)=\"Mapping\"{}\n",
	"			simple-map(String)=\"'1'='DAEMON START','2'='DAEMON STOP','3'='CHECK OUT','4'='CHECK IN','6'='TIMED OUT','7'='FEATURE INCREMENT','8'='USAGE TRACE ON','9'='USAGE TRACE OFF','10'='USAGE NOT FOUND','14'='FALSE USAGE','20'='DENIED','21'='UNSUPPORTED','22'='QUEUED'\"{}\n",
	"			source(String)=\"status\"{}\n",
	"			target(String)=\"event_type\"{}\n",
	"		}\n",	
	);
	
	&_add_filter_process( \@lines, \@insert_def , "Threshold:remove-empty-features" , "Mapping:status" , 6 );
	
	
	my $ind = 0 ;
	
	if ( grep {/Mapping.license-manager/} @lines )
	{
		$ind++ while(@lines[$ind] !~ /Threshold.empty-event/ );
		$ind+=2;
		if ( $lines[$ind] =~ /Mapping.license-manager/ )
		{
			splice @lines, $ind , 2  ;
			$ind++ while(@lines[$ind] !~ /filter-defs/ );
			splice @lines, $ind - 1 , 1  ;
			
			$ind++ while(@lines[$ind] !~ /Mapping.license-manager/ );
			splice @lines, $ind , 8  ;
		}
	}
	
	
	
	
	
	&_update_filter_definition( \@lines , "Summarize:count.compare" ,
		"\t\t\tcompare(String)=\"start,product,package,feature,feature_version,user,host,event_type,event_subtype,data_type,details\"{}" );
	
	&_update_filter_definition( \@lines , "Report.classifications" ,
		"\t\t\tclassifications(String)=\"interval:start:'104':'0':product:'0':package:'0':feature:'0':feature_version:'0':used_licenses:'0':event_type:'0':event_subtype:'0':user:'0':host:'0':details\"{}" );	
	
	&_update_filter_definition( \@lines , "Mapping:fix-valid-empty-feature.simple-map" ,
		"\t\t\tsimple-map(String)=\"'lumevents'='LUM server', 'pelotonevents'='PELOTON server'\"{}" );
		
		
		
	_save_filter( $cfg ,@lines );
}

sub _update_licenseevents_usergroups
{
	my $cfg = $filtrator_dir . "/licenseevents-usergroups.conf" ;
	my @lines = _read_file( $cfg );
	
	&_grep_and_replace( \@lines , "Concurrent.*", "Concurrent:feature()=\"\"" );
	
	&_add_filter_process( \@lines, \@insert_threshold_zero_duration , "" , "Threshold:zero-duration" , 1 );
	&_add_filter_process( \@lines, \@insert_duplicate_license_usage , "CalcSum:feature" , "Duplicate:license-usage" , 4 );
	&_add_filter_process( \@lines, \@insert_duplicate_usergroups , "Duplicate:license-usage" , "Duplicate:usergroups" , 5 );

	my $ind = 0 ;
	
	if ( grep {/Mapping.usergroups/} @lines )
	{
		$ind++ while(@lines[$ind] !~ /GenerateFields.elapsed.time/ );
		$ind+=2;
		if ( $lines[$ind] =~ /Mapping.usergroups/ )
		{
			splice @lines, $ind , 2  ;
			$ind++ while(@lines[$ind] !~ /filter-defs/ );
			splice @lines, $ind - 1 , 1  ;
			
			$ind++ while(@lines[$ind] !~ /Mapping.usergroups/ );
			splice @lines, $ind , 8  ;
		}
	
	}
	
	if ( grep {/GenerateFields.utilization/} @lines )
	{
		$ind = 0;
		$ind++ while(@lines[$ind] !~ /GenerateFields.utilization/ );
		splice @lines, $ind , 10  ;
	}
	
	&_update_filter_definition( \@lines , "Concurrent:feature.filter" ,
		"\t\t\tfilter(String)=\"Concurrent\"{}" );
	&_update_filter_definition( \@lines , "Concurrent:feature.zero-count" ,
		"\t\t\tzero-count(Bool)=\"false\"{}" );
	&_update_filter_definition( \@lines , "Concurrent:feature.compare" ,
		"\t\t\tcompare(String)=\"status,product,feature,feature_version,var8,var9\"{}" );
	
	
	if ( !  grep {/keytype/} @lines )
	{
		$ind = 0;
		$ind++ while(@lines[$ind] !~ /filter.defs/ );
		$ind++ while(@lines[$ind] !~ /Concurrent.feature/ );
		$ind++ while(@lines[$ind] !~ /count/ );
		$ind++;
		
		if ( @lines[$ind] !~  /keytype/ ) {
			splice @lines , $ind , 0 , "\t\t\tkeytype(String)=\"normal+:var8,var9\"{}\n";
		}
	}
	
	&_update_filter_definition( \@lines , "Report:usergroup-license-usage.classifications" ,
		"\t\t\tclassifications(String)=\"interval:start:'91':'0':product:'0':feature:'0':feature_version:'0':primetime:'0':var8:'0':var9:'0'\"{}" );
	
	&_update_filter_definition( \@lines , "Report:usergroup-license-usage.measurements" ,
		"\t\t\tmeasurements(String)=\"'1':elapsed_time:avg:max:min\"{}" );
		
	&_update_filter_definition( \@lines , "Summarize:license-usage.compare" ,
		"\t\t\tcompare(String)=\"start,product,feature,feature_version,var8,var9\"{}" );
		
	&_update_filter_definition( \@lines , "GenerateFields:elapsed-time.fields.elapsed_time" ,
		"\t\t\t\telapsed_time(String)=\"var3=duration*used_licenses\"{}" );
		
	&_update_filter_definition( \@lines , "CalcSum:product.remove" ,
		"\t\t\t\tremove(String)=\"\"{}" );
	
	_save_filter( $cfg ,@lines );
	
}

sub _update_licenseevents_individual_usage
{
	my $cfg = $filtrator_dir . "/licenseevents-individual-usage.conf" ;
	my @lines = _read_file( $cfg );
	
	my $ind = 0;
	&_add_filter_process( \@lines, \@insert_threshold_zero_duration , "" , "Threshold:zero-duration" , 1 );
	
	my @insert_def = (
	"		Duplicate:license-usage()=\"\"\n",
	"		{\n",
	"			filter(String)=\"Duplicate\"{}\n",
	"			global()=\"\"\n",
	"			{\n",
	"				set-values(String)=\"feature_version='all'\"{}\n",
	"			}\n",
	"		}\n"
	);
	
	&_add_filter_process( \@lines, \@insert_def , "CalcSum:feature" , "Duplicate:license-usage" , 5 );
	&_add_filter_process( \@lines, \@insert_duplicate_usergroups , "Duplicate.license.usage" , "Duplicate:usergroups" , 5 );
	
	
	if ( grep {/Mapping.user-name/} @lines )
	{
		$ind++ while(@lines[$ind] !~ /GenerateFields.elapsed-time/ );
		$ind+=2;
		if ( $lines[$ind] =~ /Mapping.user-name/ )
		{
			splice @lines, $ind , 2  ;
			$ind++ while(@lines[$ind] !~ /filter-defs/ );
			splice @lines, $ind - 1 , 1  ;
			
			$ind++ while(@lines[$ind] !~ /Mapping.user-name/ );
			splice @lines, $ind , 8  ;
		}
	
	}
	
	
	if ( !  grep {/keytype/} @lines )
	{
		$ind = 0;
		$ind++ while(@lines[$ind] !~ /filter.defs/ );
		$ind++ while(@lines[$ind] !~ /Concurrent.feature/ );
		$ind++ while(@lines[$ind] !~ /count/ );
		$ind++;
		
		if ( @lines[$ind] !~  /keytype/ ) {
			splice @lines , $ind , 0 , "\t\t\tkeytype(String)=\"normal+:var8,var9\"{}\n";
		}
	}
	
	if ( grep {/GenerateFields.utilization/} @lines )
	{
		$ind = 0;
		$ind++ while(@lines[$ind] !~ /GenerateFields.utilization/ );
		splice @lines, $ind , 10  ;
	}
	
	
	&_update_filter_definition( \@lines , "Report:individual-license-usage.classifications" ,
		"\t\t\tclassifications(String)=\"interval:start:'90':'0':product:'0':feature:'0':feature_version:'0':primetime:'0':var8:'0':var9:'0':user\"{}" );
	
	&_update_filter_definition( \@lines , "GenerateFields:elapsed-time.fields.elapsed_time" ,
		"\t\t\t\telapsed_time(String)=\"var3=duration*used_licenses\"{}" );
	
	&_update_filter_definition( \@lines , "RemoveClass:feature.remove" ,
		"\t\t\tremove(String)=\"status,product,feature,feature_version,user,host,tty,licenses,used_licenses\"{}" );
	
	&_update_filter_definition( \@lines , "RemoveClass:feature.compare" ,
		"\t\t\tcompare(String)=\"status,product,feature,feature_version,user,host,tty,licenses,used_licenses\"{}" );
		
	&_update_filter_definition( \@lines , "Concurrent:feature.count" ,
		"\t\t\tcount(String)=\"used_licenses\"{}" );
		
	&_update_filter_definition( \@lines , "Concurrent:feature.zero-count" ,
		"\t\t\tzero-count(Bool)=\"false\"{}" );
	
	_save_filter( $cfg ,@lines );
}

sub _update_licenseevents_host_usage
{
	my $cfg =  $filtrator_dir . "/licenseevents-host-usage.conf";
	my @lines = _read_file( $cfg );
	&_grep_and_replace( \@lines , "Report.individual-license-usage", "Report:host-license-usage" );
	&_update_filter_definition( \@lines , "Concurrent:feature.zero-count" ,
		"\t\t\t\t\t\tzero-count(Bool)=\"false\"{}" );
	
	_save_filter( $cfg ,@lines );
}

sub _update_licenseevents_hostgroups
{
	my $cfg = $filtrator_dir . "/licenseevents-hostgroups.conf";
	my @lines = _read_file( $cfg );
	
	&_grep_and_replace( \@lines , "licenseevents-usergroups", "licenseevents-hostgroups" );
	&_grep_and_replace( \@lines , "Report.usergroup-license-usage", "Report:hostgroup-license-usage" );
	
	&_update_filter_definition( \@lines , "Report:hostgroup-license-usage.classifications" ,
		"\t\t\t\t\t\tclassifications(String)=\"interval:start:'109':'0':product:'0':feature:'0':feature_version:'0':primetime:'0':var8:'0':var9:'0'\"{}" );
		
	&_update_filter_definition( \@lines , "Concurrent:feature.zero-count" ,
		"\t\t\t\t\t\tzero-count(Bool)=\"false\"{}" );

	_save_filter( $cfg ,@lines );
}


sub _update_licenseevents_total_usage
{
	my $cfg =  $filtrator_dir . "/licenseevents-total-usage.conf";
	my @lines = _read_file( $cfg );
	
	&_add_filter_process( \@lines, \@insert_threshold_zero_duration , "" , "Threshold:zero-duration" , 1 );
	
	my $ind = 0 ;
	
	$ind++ while(@lines[$ind] !~ /CalcSum.feature/ );
	$ind+=2;
	if ( $lines[$ind] =~ /Concurrent.feature/ )
	{
		splice @lines, $ind , 2  ;
		$ind++ while(@lines[$ind] !~ /CalcSum.license-usage-all/ );
		splice @lines, $ind + 2 , 0 , "\t\t\t\t\t\t\t\t\tConcurrent:feature()=\"\"\n";
		splice @lines, $ind + 3 , 0 , "\t\t\t\t\t\t\t\t\t{\n";
		
	}
	
	&_update_filter_definition( \@lines , "CalcSum:product.compare" ,
		"\t\t\tcompare(String)=\"status,product,feature,feature_version,user,host,tty,licenses,used_licenses\"" );
		
	&_update_filter_definition( \@lines , "CalcSum:product.remove" ,
		"\t\t\tremove(String)=\"\"{}" );
		
	&_update_filter_definition( \@lines , "CalcSum:feature.compare" ,
		"\t\t\tcompare(String)=\"status,product,feature,feature_version,user,host,tty,licenses,used_licenses\"");
		
	&_update_filter_definition( \@lines , "RemoveClass:feature.remove" ,
		"\t\t\tremove(String)=\"status,product,feature,feature_version,user,host,tty,licenses,used_licenses\"{}");
		
	&_update_filter_definition( \@lines , "RemoveClass:feature.compare" ,
		"\t\t\tcompare(String)=\"status,product,feature,feature_version,user,host,tty,licenses,used_licenses\"{}");
		
	&_update_filter_definition( \@lines , "Concurrent:feature.compare" ,
		"\t\t\tcompare(String)=\"status,product,feature,feature_version,licenses,used_licenses\"{}");
		
	&_update_filter_definition( \@lines , "GenerateFields:elapsed-time.fields.elapsed_time" ,
		"\t\t\t\telapsed_time(String)=\"var3=duration\*used_licenses\"{}");
		
	&_update_filter_definition( \@lines , "Report:total-license-usage.classifications" ,
		"\t\t\tclassifications(String)=\"interval:start:'89':'0':product:'0':feature:'0':feature_version:'0':licenses:'0':used_licenses:'0':primetime:'0':daemon_status:'0'\"{}" );
	
	&_update_filter_definition( \@lines , "CalcSum:license-usage-all.compare" ,
		"\t\t\tcompare(String)=\"start,product,feature,feature_version,origsource\"" );
	
	&_update_filter_definition( \@lines , "CalcSum:license-usage-all.remove" ,
		"\t\t\tremove(String)=\"feature_version,origsource\"{}" );
	
	_save_filter( $cfg ,@lines );
}

sub _get_filter_process
{
	my ( $lines ) = @_;
	my $ind = 0;
	$ind++ while( @$lines[ $ind ] !~ /filter-process/ );
	$ind++;
	my @filter_process = ();
	while( @$lines[ $ind ] !~ /filter-defs/ )
	{	
		if ( @$lines[$ind] =~/.*()=/ )
		{
			
			my $filtername = @$lines[$ind];
			$filtername =~ s/^\s+|\s+$//g;
			$filtername =~ s/\(\)=\"\"//g;
			 
			push @filter_process , $filtername;
		}
		$ind++;
	}

	return \@filter_process;
}

sub _add_filter_process
{
	my ( $lines, $insert_def, $filter_path, $name ,$tabs ) = @_;
	
	if ( ! grep {/$name/} @$lines ) 
	{
		DEBUG ( "Adding Filter process '$name'" );
		my $ind = 0;
		my @path = split '\.', $filter_path ;
		
		$ind++ while(@$lines[$ind] !~ /filter-process/ );
		
		foreach my $name (@path)
		{
			foreach ( @$lines )
			{
				last if ( @$lines[$ind] =~ /$name/ );
				$ind++;
			}
		}
		my $tab;
		for (my $i=0; $i <= $tabs; $i++) {
		  $tab .= "\t";
		}
		$ind+= 2;
		splice @$lines , $ind, 0 , "$tab$name()=\"\"\n" ;
		$ind++;
		splice @$lines , $ind , 0 , "$tab\{\n" ;
		
		$ind++;
		
		# fix tabbings
		while(@$lines[$ind] !~ /filter-defs/ )
		{
			@$lines[$ind] = "\t" . @$lines[$ind];
			$ind++;
		}
		
		splice @$lines , $ind , 0 , "\t\}\n" ;
		
		&_add_filter_def ( \@$lines, \@$insert_def , $name );
	}
	else
	{
		DEBUG ( "Filter process '$name' already exist" );
	}
}
sub _read_file
{
	my ($cfg) = @_;
	VERBOSE( "Reading $cfg " );
	open(IN, "$cfg") || die "Error: Unable to read cfg '$cfg'!\n";
	my @lines = <IN>;
	close(IN);
	return @lines;
}

sub _add_filter_def
{
	my ( $lines, $insert_def, $name ) = @_;
	my $ind = 0;
	$ind++ while( @$lines[$ind] !~ /filter-defs/ );
	splice @$lines , $ind + 2 , 0 , @$insert_def ;
}
sub _update_filter_definition
{
	
	my ( $lines, $filter_path, $new_value ) = @_;	
	my $ind = 0;
	my @path = split '\.', $filter_path ;
	
	$ind++ while(@$lines[$ind] !~ /filter-defs/ );
	
	foreach my $name (@path) {
		#$ind++ while(@$lines[$ind] !~ /$name/ );
		foreach ( @$lines )
		{
			last if ( @$lines[$ind] =~ /$name/ );
			$ind++;
		}
	}
	DEBUG ("Updating filter-defs value '$filter_path'");
	#@$lines[$ind] = s/@$lines[$ind]/$new_value/g
	@$lines[$ind] = "$new_value\n"; 	
}
sub _grep_and_replace
{
	my ( $lines,  $grep_string, $new_value ) = @_;
	s/$grep_string/$new_value/g foreach(@$lines);
}

sub _save_filter
{
	my ( $cfg, @lines ) = @_; 
	
	my $old = "$cfg.old";
	my $new = "$cfg.new";
	
	open(OUT, ">$new") || die "Error: Can't write new cfg '$new'!\n";
	print OUT @lines;
	close(OUT);

	if( ! rename($cfg, $old)) {
		die "Error: Unable to previous config to '$old'!\n";
	}

	if( ! rename($new, $cfg)) {
		die "Error: Unable to rename new config to '$cfg'!\n";
	}
}

sub _get_config_value
{
	my ( $key ) = @_;
	my $value =  ` . /tmp/openit-cfg-shenv && echo \$$key`;	
	DEBUG ( "Key: $key -> Value: $value" );
	chomp $value;
	return $value;
}

sub DEBUG
{
	if ( defined $opt_debug )
	{
		my ($message) = @_;
		chomp $message;
		print "DEBUG: " . $message . "\n";
	}
}

sub VERBOSE
{
	my ($message) = @_;
	print " -- " . $message . "\n" ;
}